@@ -561,8 +561,8 @@ fn test_ls_a() {
         .arg("-a")
         .arg("-1")
         .succeeds()
-        .stdout_contains(&".test-1")
-        .stdout_contains(&"..")
+        .stdout_contains(".test-1")
+        .stdout_contains("..")
         .stdout_matches(&re_pwd);
 
     scene
@@ -590,8 +590,8 @@ fn test_ls_a() {
         .arg("-1")
         .arg("some-dir")
         .succeeds()
-        .stdout_contains(&".test-2")
-        .stdout_contains(&"..")
+        .stdout_contains(".test-2")
+        .stdout_contains("..")
         .no_stderr()
         .stdout_matches(&re_pwd);
 
@@ -966,7 +966,7 @@ fn test_ls_long() {
         result.stdout_matches(&Regex::new(r"[-bcCdDlMnpPsStTx?]([r-][w-][xt-]){3}.*").unwrap());
 
         #[cfg(windows)]
-        result.stdout_contains("---------- 1 somebody somegroup");
+        result.stdout_matches(&Regex::new(r"[-dl](r[w-]x){3}.*").unwrap());
     }
 }
 
@@ -1174,7 +1174,7 @@ fn test_ls_long_symlink_color() {
                 captures.get(1).unwrap().as_str().to_string(),
                 captures.get(2).unwrap().as_str().to_string(),
             ),
-            None => ("".to_string(), input.to_string()),
+            None => (String::new(), input.to_string()),
         }
     }
 
@@ -1821,7 +1821,7 @@ fn test_ls_files_dirs() {
         .ucmd()
         .arg("doesntexist")
         .fails()
-        .stderr_contains(&"'doesntexist': No such file or directory");
+        .stderr_contains("'doesntexist': No such file or directory");
 
     // One exists, the other doesn't
     scene
@@ -1829,8 +1829,8 @@ fn test_ls_files_dirs() {
         .arg("a")
         .arg("doesntexist")
         .fails()
-        .stderr_contains(&"'doesntexist': No such file or directory")
-        .stdout_contains(&"a:");
+        .stderr_contains("'doesntexist': No such file or directory")
+        .stdout_contains("a:");
 }
 
 #[test]
@@ -1851,7 +1851,7 @@ fn test_ls_recursive() {
         .arg("z")
         .arg("-R")
         .succeeds()
-        .stdout_contains(&"z:");
+        .stdout_contains("z:");
     let result = scene
         .ucmd()
         .arg("--color=never")
@@ -1861,9 +1861,9 @@ fn test_ls_recursive() {
         .succeeds();
 
     #[cfg(not(windows))]
-    result.stdout_contains(&"a/b:\nb");
+    result.stdout_contains("a/b:\nb");
     #[cfg(windows)]
-    result.stdout_contains(&"a\\b:\nb");
+    result.stdout_contains("a\\b:\nb");
 }
 
 #[test]
@@ -2014,7 +2014,7 @@ fn test_ls_indicator_style() {
         "-p",
     ] {
         // Verify that classify and file-type both contain indicators for symlinks.
-        scene.ucmd().arg(opt).succeeds().stdout_contains(&"/");
+        scene.ucmd().arg(opt).succeeds().stdout_contains("/");
     }
 
     // Classify, Indicator options should not contain any indicators when value is none.
@@ -2030,9 +2030,9 @@ fn test_ls_indicator_style() {
             .ucmd()
             .arg(opt)
             .succeeds()
-            .stdout_does_not_contain(&"/")
-            .stdout_does_not_contain(&"@")
-            .stdout_does_not_contain(&"|");
+            .stdout_does_not_contain("/")
+            .stdout_does_not_contain("@")
+            .stdout_does_not_contain("|");
     }
 
     // Classify and File-Type all contain indicators for pipes and links.
@@ -2043,8 +2043,8 @@ fn test_ls_indicator_style() {
             .ucmd()
             .arg(format!("--indicator-style={}", opt))
             .succeeds()
-            .stdout_contains(&"@")
-            .stdout_contains(&"|");
+            .stdout_contains("@")
+            .stdout_contains("|");
     }
 
     // Test sockets. Because the canonical way of making sockets to test is with
@@ -2057,7 +2057,7 @@ fn test_ls_indicator_style() {
             .tempdir()
             .expect("failed to create dir");
         let socket_path = dir.path().join("sock");
-        let _listener = UnixListener::bind(&socket_path).expect("failed to create socket");
+        let _listener = UnixListener::bind(socket_path).expect("failed to create socket");
 
         new_ucmd!()
             .args(&[
@@ -2093,13 +2093,13 @@ fn test_ls_indicator_style() {
             .ucmd()
             .arg(format!("--indicator-style={}", opt))
             .succeeds()
-            .stdout_contains(&"/");
+            .stdout_contains("/");
     }
 
     // Same test as above, but with the alternate flags.
     let options = vec!["--classify", "--file-type", "-p"];
     for opt in options {
-        scene.ucmd().arg(opt).succeeds().stdout_contains(&"/");
+        scene.ucmd().arg(opt).succeeds().stdout_contains("/");
     }
 
     // Classify and File-Type all contain indicators for pipes and links.
@@ -2110,7 +2110,7 @@ fn test_ls_indicator_style() {
             .ucmd()
             .arg(format!("--indicator-style={}", opt))
             .succeeds()
-            .stdout_contains(&"@");
+            .stdout_contains("@");
     }
 }
 
@@ -2631,7 +2631,7 @@ fn test_ls_ignore_hide() {
         .arg("--ignore=READ[ME")
         .arg("-1")
         .succeeds()
-        .stderr_contains(&"Invalid pattern")
+        .stderr_contains("Invalid pattern")
         .stdout_is("CONTRIBUTING.md\nREADME.md\nREADMECAREFULLY.md\nsome_other_file\n");
 
     scene
@@ -2639,7 +2639,7 @@ fn test_ls_ignore_hide() {
         .arg("--hide=READ[ME")
         .arg("-1")
         .succeeds()
-        .stderr_contains(&"Invalid pattern")
+        .stderr_contains("Invalid pattern")
         .stdout_is("CONTRIBUTING.md\nREADME.md\nREADMECAREFULLY.md\nsome_other_file\n");
 }
 
@@ -3195,7 +3195,7 @@ fn test_ls_context_format() {
     ] {
         let format = format!("--format={}", word);
         ts.ucmd()
-            .args(&[&"-Z", &format.as_str(), &"/"])
+            .args(&["-Z", format.as_str(), "/"])
             .succeeds()
             .stdout_only(
                 unwrap_or_return!(expected_result(&ts, &["-Z", format.as_str(), "/"])).stdout_str(),
@@ -3248,6 +3248,7 @@ fn test_ls_multiple_a_A() {
 }
 
 #[test]
+#[cfg(feature = "ln")]
 fn test_ls_quoting() {
     let scene = TestScenario::new(util_name!());
 
@@ -3267,6 +3268,7 @@ fn test_ls_quoting() {
 }
 
 #[test]
+#[cfg(feature = "ln")]
 fn test_ls_quoting_color() {
     let scene = TestScenario::new(util_name!());
 
