@@ -35,27 +35,20 @@ TOTAL_ORDER=$(echo -e $PARTIAL_ORDER | tsort | tac)
 TOTAL_ORDER=${TOTAL_ORDER#ROOT}
 
 set -e
-# for dir in src/uu/stdbuf/src/libstdbuf/; do
-#     (
-#         cd "$dir"
-#         #shellcheck disable=SC2086
-#         cargo publish $ARG
-#     )
-#     sleep 10s
-# done
+for dir in src/uucore_procs/ src/uucore src/uu/stdbuf/src/libstdbuf/; do
+    (
+        cd "$dir"
+        #shellcheck disable=SC2086
+        cargo publish $ARG || true
+    )
+done
 
-SEEN_CHROOT=false;
 for p in $TOTAL_ORDER; do
-    if [ $p == "src/uu/chroot/" ]; then
-        SEEN_CHROOT=true
-    fi
-    if $SEEN_CHROOT; then
-        (
-            cd "src/uu/$p"
-            #shellcheck disable=SC2086
-            cargo publish $ARG
-        )
-    fi
+    (
+        cd "src/uu/$p"
+        #shellcheck disable=SC2086
+        cargo publish $ARG || true
+    )
 done
 
 #shellcheck disable=SC2086
